/*  
 * Copyright (c) 2010 Ant Kutschera, maxant
 * 
 * This file is part of the maxant Blog.
 * 
 * maxant Blog is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant Blog is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with maxant Blog.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.co.maxant.wsdemo;

import java.text.SimpleDateFormat;

import javax.jws.HandlerChain;
import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * problem: validation isnt working.<br>
 * <br>
 * for example, a required date field can have an invalid date set, and it is simply passed to the webservice method
 * as null, even though its required.  one might expect JAX-WS to automatically validate the body of the
 * incoming SOAP envelope against the XSD defined in the WSDL, but it doesnt.  from the jax-ws 2.0 final release specs,
 * chapter 1.1:<br>
 * <br>
 * <i>
 * JAXB 2.0[10] will add support for Java to XML mapping, additional support for less used XML
 * schema constructs, and provide bidirectional customization of Java , XML data binding. JAXWS
 * 2.0 will allow full use of JAXB provided facilities including binding customization and optional
 * schema validation.</i><br>
 * <br>
 * this means, invalid elements of a message are simply not parsed, the resulting unmarshalled objects 
 * contain null references to those attributes (even if they are defined as being required!) and 
 * your web service is called, when you would expect it not to be.<br>
 * <br>
 * the solution is to set up a logical handler which uses JAXP to do the validation against the schema.<br>
 * <br>
 * in websphere, which appears to be based on axis2, the validation works for an initially wrong body. however
 * if that problem is corrected and the next request is valid, then subsequent invalid requests are not validated
 * properly.  the faulty elements are simply ignored!<br>
 * <br>
 * this is a test on GFv3 to see if the problems happen here too.<br>
 * <br>
 * steps to recreate:<br>
 * <ul>
 * 	<li>set attribute on object class to be "required"</li>
 * 	<li>send a good request using a test client, and monitor (record) it. expect a good response.</li>
 * 	<li>modify the body of the request, making the date invalid, and adjust the header content length!</li>
 * 	<li>resend the request, expect an error with the correct message</li>
 * 	<li>resend the good request, expect a good response.</li>
 * 	<li>resend the bad request, expect an error with the wrong mesage</li>
 * </ul>
 * The problem is not recreatable on glassfish!<br>
 * <br>
 * Anyway, important point, is that if you want to implement validation against the schema, then 
 * you need to do it yourself, using a logical handler.<br>
 * <br>
 * Generated clients help validation by ensuring for example that method parameters are of the correct
 * type (e.g. a Date), but they don't help against the caller leaving out elements or attributes. besides
 * you cannot guarantee that a client will behave.  for all you know the caller is building up their soap
 * body manually using strings and sending it over the wire as an http request with something like 
 * apache HTTPClient!  crazy, but not impossible.  so the only way to guarantee the incoming message is 
 * a valid one, is to validate against the schema, as shown here, using a {@link HandlerChain}.
 */
@WebService
@HandlerChain(file="handler.xml") //configure the handlers in this file
public class GislerService {

	private static final String FORMAT_DD_MMM_YYYY_HH_MM_SS = "dd. MMM yyyy HH:mm:ss";

	/**
	 * some arbitrary business logic that we offer to our clients...
	 * @param om an object model to parse
	 * @return a formatted date, based on the input.
	 */
	@WebMethod
	public String formatDate(ObjectModel om){
		SimpleDateFormat sdf = new SimpleDateFormat(FORMAT_DD_MMM_YYYY_HH_MM_SS);
		return "Formatted, its " + sdf.format(om.date);
	}

}
