/*  
 * Copyright (c) 2010 Ant Kutschera, maxant
 * 
 * This file is part of the maxant Blog.
 * 
 * maxant Blog is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant Blog is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with maxant Blog.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.co.maxant.wsdemo;

import static org.junit.Assert.*;

import java.util.Map;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.ws.BindingProvider;

import org.junit.Test;

/**
 * junit4 test class.
 */
public class Tester {

	/**
	 * this test simply checks the repeatability of sending a valid request, and an invalid request.
	 * to make it invalid, we simply remove the required field. it would be nice to also have a test
	 * which passes an invalid date string, but we cant do that using the generated client, because its
	 * strongly typed and only accepts an {@link XMLGregorianCalendar}.  to do these more complex tests,
	 * we use the TCP/IP monitor in eclipse, or a tool like SOAPUI.
	 */
	@Test
	public void test() {
		GislerServiceService service = new GislerServiceService();
		GislerService gs = service.getGislerServicePort();

        //use monitored port, rather than web server port.
		String url = "http://localhost:8085/Gisler/GislerServiceService";
		Map<String, Object> map = ((BindingProvider)gs).getRequestContext();
		map.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, url);

		ObjectModel om = new ObjectModel();
		om.setName("myName");

		XMLGregorianCalendar c = null;

		//valid call
		try{
			DatatypeFactory dtf = DatatypeFactory.newInstance();
			c = dtf.newXMLGregorianCalendar();
			c.setSecond(0);
			c.setMinute(0);
			c.setHour(0);
			c.setDay(1);
			c.setMonth(1);
			c.setYear(5000);
			om.setDate(c);
			String s = gs.formatDate(om);
			assertEquals("Formatted, its 01. Jan 5000 00:00:00", s);
		}catch(Exception e){
			e.printStackTrace();
			fail(e.getMessage());
		}

		//invalid call
		try{
			om.setDate(null); //causes exception because its REQUIRED!
			gs.formatDate(om);
			fail(""); //expecting an exception, because date is missing!
		}catch(Exception e){
			//all is good!
			assertEquals("cvc-complex-type.2.4.a: Invalid content was found starting with element 'name'. One of '{date}' is expected.", e.getMessage());
		}
		
		//another valid call
		try{
			c.setYear(6000);
			om.setDate(c);
			om.setOptionalDate(c);
			String s = gs.formatDate(om);
			assertEquals("Formatted, its 01. Jan 6000 00:00:00", s);
		}catch(Exception e){
			e.printStackTrace();
			fail(e.getMessage());
		}

		//another invalid call
		try{
			om.setDate(null); //causes exception because its REQUIRED!
			om.setOptionalDate(c);
			gs.formatDate(om);
			fail(""); //expecting an exception, because date is missing!
		}catch(Exception e){
			//all is good!
			assertEquals("cvc-complex-type.2.4.a: Invalid content was found starting with element 'name'. One of '{date}' is expected.", e.getMessage());
		}
		
	}
}
